@@ -1,9 +1,33 @@
+from __future__ import unicode_literals
+
 import copy
 from django.db import models
+from django.conf import settings
 from django.contrib import admin
-from django.contrib.auth.models import User
 from django.utils import importlib
-from manager import HistoryDescriptor
+from .manager import HistoryDescriptor
+
+try:
+    from django.utils.encoding import python_2_unicode_compatible
+except ImportError:  # django 1.3 compatibility
+    import sys
+
+    # copy of django function without use of six
+    def python_2_unicode_compatible(klass):
+        """
+        A decorator that defines __unicode__ and __str__ methods under Python 2.
+        Under Python 3 it does nothing.
+
+        To support Python 2 and 3 with a single code base, define a __str__ method
+        returning text and apply this decorator to the class.
+        """
+        if sys.version_info[0] != 3:
+            klass.__unicode__ = klass.__str__
+            klass.__str__ = lambda self: self.__unicode__().encode('utf-8')
+        return klass
+
+
+registered_models = {}
 
 
 class HistoricalRecords(object):
@@ -15,8 +39,10 @@ class HistoricalRecords(object):
         def save_without_historical_record(self, *args, **kwargs):
             """Caution! Make sure you know what you're doing before you use this method."""
             self.skip_history_when_saving = True
-            ret = self.save(*args, **kwargs)
-            del self.skip_history_when_saving
+            try:
+                ret = self.save(*args, **kwargs)
+            finally:
+                del self.skip_history_when_saving
             return ret
         setattr(cls, 'save_without_historical_record', save_without_historical_record)
 
@@ -42,12 +68,23 @@ class HistoricalRecords(object):
         """
         attrs = {'__module__': self.module}
 
+        app_module = '%s.models' % model._meta.app_label
+        if model.__module__ != self.module:
+            # registered under different app
+            attrs['__module__'] = self.module
+        elif app_module != self.module:
+            # has meta options with app_label
+            app = models.get_app(model._meta.app_label)
+            attrs['__module__'] = app.__name__ # full dotted name
+
         fields = self.copy_fields(model)
         attrs.update(fields)
         attrs.update(self.get_extra_fields(model, fields))
-        attrs.update(Meta=type('Meta', (), self.get_meta_options(model)))
+        # type in python2 wants str as a first argument
+        attrs.update(Meta=type(str('Meta'), (), self.get_meta_options(model)))
         name = 'Historical%s' % model._meta.object_name
-        return type(name, (models.Model,), attrs)
+        registered_models[model._meta.db_table] = model
+        return python_2_unicode_compatible(type(str(name), (models.Model,), attrs))
 
     def copy_fields(self, model):
         """
@@ -115,7 +152,7 @@ class HistoricalRecords(object):
         return {
             'history_id': models.AutoField(primary_key=True),
             'history_date': models.DateTimeField(auto_now_add=True),
-            'history_user': models.ForeignKey(User, null=True),
+            'history_user': models.ForeignKey(getattr(settings, 'AUTH_USER_MODEL', 'auth.User'), null=True),
             'history_type': models.CharField(max_length=1, choices=(
                 ('+', 'Created'),
                 ('~', 'Changed'),
@@ -124,8 +161,8 @@ class HistoricalRecords(object):
             'history_object': HistoricalObjectDescriptor(model),
             'instance': property(get_instance),
             'revert_url': revert_url,
-            '__unicode__': lambda self: u'%s as of %s' % (self.history_object,
-                                                          self.history_date)
+            '__str__': lambda self: '%s as of %s' % (self.history_object,
+                                                     self.history_date)
         }
 
     def get_meta_options(self, model):
